@@ -13,8 +13,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import json
+import os
 import sys
 
+from oslo_utils import strutils
+
 from glanceclient._i18n import _
 from glanceclient.common import progressbar
 from glanceclient.common import utils
@@ -25,8 +29,6 @@ from glanceclient.v2 import images
 from glanceclient.v2 import namespace_schema
 from glanceclient.v2 import resource_type_schema
 from glanceclient.v2 import tasks
-import json
-import os
 
 MEMBER_STATUS_VALUES = image_members.MEMBER_STATUS_VALUES
 IMAGE_SCHEMA = None
@@ -49,7 +51,17 @@ def get_image_schema():
 @utils.schema_args(get_image_schema, omit=['created_at', 'updated_at', 'file',
                                            'checksum', 'virtual_size', 'size',
                                            'status', 'schema', 'direct_url',
-                                           'locations', 'self'])
+                                           'locations', 'self', 'os_hidden',
+                                           'os_hash_value', 'os_hash_algo'])
+# NOTE(rosmaita): to make this option more intuitive for end users, we
+# do not use the Glance image property name 'os_hidden' here.  This means
+# we must include 'os_hidden' in the 'omit' list above and handle the
+# --hidden argument by hand
+@utils.arg('--hidden', type=strutils.bool_from_string, metavar='[True|False]',
+           default=None,
+           dest='os_hidden',
+           help=("If true, image will not appear in default image list "
+                 "response."))
 @utils.arg('--property', metavar="<key=value>", action='append',
            default=[], help=_('Arbitrary property to associate with image.'
                               ' May be used multiple times.'))
@@ -59,11 +71,15 @@ def get_image_schema():
                   'passed to the client via stdin.'))
 @utils.arg('--progress', action='store_true', default=False,
            help=_('Show upload progress bar.'))
+@utils.arg('--backend', metavar='<STORE>',
+           default=utils.env('OS_IMAGE_BACKEND', default=None),
+           help='Backend store to upload image to.')
 @utils.on_data_require_fields(DATA_FIELDS)
 def do_image_create(gc, args):
     """Create a new image."""
     schema = gc.schemas.get("image")
     _args = [(x[0].replace('-', '_'), x[1]) for x in vars(args).items()]
+
     fields = dict(filter(lambda x: x[1] is not None and
                          (x[0] == 'property' or
                           schema.is_core_property(x[0])),
@@ -74,13 +90,25 @@ def do_image_create(gc, args):
         key, value = datum.split('=', 1)
         fields[key] = value
 
+    backend = args.backend
+
     file_name = fields.pop('file', None)
+    using_stdin = not sys.stdin.isatty()
+    if args.backend and not (file_name or using_stdin):
+        utils.exit("--backend option should only be provided with --file "
+                   "option or stdin.")
+
+    if backend:
+        # determine if backend is valid
+        _validate_backend(backend, gc)
+
     if file_name is not None and os.access(file_name, os.R_OK) is False:
         utils.exit("File %s does not exist or user does not have read "
                    "privileges to it" % file_name)
     image = gc.images.create(**fields)
     try:
         if utils.get_data_file(args) is not None:
+            backend = fields.get('backend', None)
             args.id = image['id']
             args.size = None
             do_image_upload(gc, args)
@@ -92,7 +120,14 @@ def do_image_create(gc, args):
 @utils.schema_args(get_image_schema, omit=['created_at', 'updated_at', 'file',
                                            'checksum', 'virtual_size', 'size',
                                            'status', 'schema', 'direct_url',
-                                           'locations', 'self'])
+                                           'locations', 'self', 'os_hidden',
+                                           'os_hash_value', 'os_hash_algo'])
+# NOTE: --hidden requires special handling; see note at do_image_create
+@utils.arg('--hidden', type=strutils.bool_from_string, metavar='[True|False]',
+           default=None,
+           dest='os_hidden',
+           help=("If true, image will not appear in default image list "
+                 "response."))
 @utils.arg('--property', metavar="<key=value>", action='append',
            default=[], help=_('Arbitrary property to associate with image.'
                               ' May be used multiple times.'))
@@ -112,6 +147,9 @@ def do_image_create(gc, args):
                   'record if no import-method and no data is supplied'))
 @utils.arg('--uri', metavar='<IMAGE_URL>', default=None,
            help=_('URI to download the external image.'))
+@utils.arg('--backend', metavar='<STORE>',
+           default=utils.env('OS_IMAGE_BACKEND', default=None),
+           help='Backend store to upload image to.')
 @utils.on_data_require_fields(DATA_FIELDS)
 def do_image_create_via_import(gc, args):
     """EXPERIMENTAL: Create a new image via image import.
@@ -132,6 +170,7 @@ def do_image_create_via_import(gc, args):
     """
     schema = gc.schemas.get("image")
     _args = [(x[0].replace('-', '_'), x[1]) for x in vars(args).items()]
+
     fields = dict(filter(lambda x: x[1] is not None and
                          (x[0] == 'property' or
                           schema.is_core_property(x[0])),
@@ -157,12 +196,21 @@ def do_image_create_via_import(gc, args):
                    "Valid values can be retrieved with import-info command." %
                    args.import_method)
 
+    # determine if backend is valid
+    backend = None
+    if args.backend:
+        backend = args.backend
+        _validate_backend(backend, gc)
+
     # make sure we have all and only correct inputs for the requested method
     if args.import_method is None:
         if args.uri:
             utils.exit("You cannot use --uri without specifying an import "
                        "method.")
     if args.import_method == 'glance-direct':
+        if backend and not (file_name or using_stdin):
+            utils.exit("--backend option should only be provided with --file "
+                       "option or stdin for the glance-direct import method.")
         if args.uri:
             utils.exit("You cannot specify a --uri with the glance-direct "
                        "import method.")
@@ -176,6 +224,9 @@ def do_image_create_via_import(gc, args):
             utils.exit("You must specify a --file or provide data via stdin "
                        "for the glance-direct import method.")
     if args.import_method == 'web-download':
+        if backend and not args.uri:
+            utils.exit("--backend option should only be provided with --uri "
+                       "option for the web-download import method.")
         if not args.uri:
             utils.exit("URI is required for web-download import method. "
                        "Please use '--uri <uri>'.")
@@ -201,12 +252,39 @@ def do_image_create_via_import(gc, args):
         utils.print_image(image)
 
 
+def _validate_backend(backend, gc):
+    try:
+        enabled_backends = gc.images.get_stores_info().get('stores')
+    except exc.HTTPNotFound:
+        # NOTE(abhishekk): To maintain backward compatibility
+        return
+
+    if backend:
+        valid_backend = False
+        for available_backend in enabled_backends:
+            if available_backend['id'] == backend:
+                valid_backend = True
+                break
+
+        if not valid_backend:
+            utils.exit("Backend '%s' is not valid for this cloud. Valid "
+                       "values can be retrieved with stores-info command." %
+                       backend)
+
+
 @utils.arg('id', metavar='<IMAGE_ID>', help=_('ID of image to update.'))
 @utils.schema_args(get_image_schema, omit=['id', 'locations', 'created_at',
                                            'updated_at', 'file', 'checksum',
                                            'virtual_size', 'size', 'status',
                                            'schema', 'direct_url', 'tags',
-                                           'self'])
+                                           'self', 'os_hidden',
+                                           'os_hash_value', 'os_hash_algo'])
+# NOTE: --hidden requires special handling; see note at do_image_create
+@utils.arg('--hidden', type=strutils.bool_from_string, metavar='[True|False]',
+           default=None,
+           dest='os_hidden',
+           help=("If true, image will not appear in default image list "
+                 "response."))
 @utils.arg('--property', metavar="<key=value>", action='append',
            default=[], help=_('Arbitrary property to associate with image.'
                               ' May be used multiple times.'))
@@ -216,6 +294,7 @@ def do_image_update(gc, args):
     """Update an existing image."""
     schema = gc.schemas.get("image")
     _args = [(x[0].replace('-', '_'), x[1]) for x in vars(args).items()]
+
     fields = dict(filter(lambda x: x[1] is not None and
                          (x[0] in ['property', 'remove_property'] or
                           schema.is_core_property(x[0])),
@@ -247,7 +326,10 @@ def do_image_update(gc, args):
            help=_("Filter images by a user-defined image property."),
            action='append', dest='properties', default=[])
 @utils.arg('--checksum', metavar='<CHECKSUM>',
-           help=_('Displays images that match the checksum.'))
+           help=_('Displays images that match the MD5 checksum.'))
+@utils.arg('--hash', dest='os_hash_value', default=None,
+           metavar='<HASH_VALUE>',
+           help=_('Displays images that match the specified hash value.'))
 @utils.arg('--tag', metavar='<TAG>', action='append',
            help=_("Filter images by a user-defined tag."))
 @utils.arg('--sort-key', default=[], action='append',
@@ -261,10 +343,27 @@ def do_image_update(gc, args):
            help=(_("Comma-separated list of sort keys and directions in the "
                    "form of <key>[:<asc|desc>]. Valid keys: %s. OPTIONAL."
                    ) % ', '.join(images.SORT_KEY_VALUES)))
+@utils.arg('--hidden',
+           dest='os_hidden',
+           metavar='[True|False]',
+           default=None,
+           type=strutils.bool_from_string,
+           const=True,
+           nargs='?',
+           help="Filters results by hidden status. Default=None.")
+@utils.arg('--include-stores',
+           metavar='[True|False]',
+           default=None,
+           type=strutils.bool_from_string,
+           const=True,
+           nargs='?',
+           help="Print backend store id.")
 def do_image_list(gc, args):
     """List images you can access."""
-    filter_keys = ['visibility', 'member_status', 'owner', 'checksum', 'tag']
+    filter_keys = ['visibility', 'member_status', 'owner', 'checksum', 'tag',
+                   'os_hidden', 'os_hash_value']
     filter_items = [(key, getattr(args, key)) for key in filter_keys]
+
     if args.properties:
         filter_properties = [prop.split('=', 1) for prop in args.properties]
         if any(len(pair) != 2 for pair in filter_properties):
@@ -295,6 +394,9 @@ def do_image_list(gc, args):
         columns += ['Disk_format', 'Container_format', 'Size', 'Status',
                     'Owner']
 
+    if args.include_stores:
+        columns += ['Stores']
+
     images = gc.images.list(**kwargs)
     utils.print_list(images, columns)
 
@@ -391,6 +493,27 @@ def do_import_info(gc, args):
         utils.print_dict(import_info)
 
 
+def do_stores_info(gc, args):
+    """Print available backends from Glance."""
+    try:
+        stores_info = gc.images.get_stores_info()
+    except exc.HTTPNotFound:
+        utils.exit('Multi Backend support is not enabled')
+    else:
+        utils.print_dict(stores_info)
+
+
+@utils.arg('--allow-md5-fallback', action='store_true',
+           default=utils.env('OS_IMAGE_ALLOW_MD5_FALLBACK', default=False),
+           help=_('If os_hash_algo and os_hash_value properties are available '
+                  'on the image, they will be used to validate the downloaded '
+                  'image data. If the indicated secure hash algorithm is not '
+                  'available on the client, the download will fail. Use this '
+                  'flag to indicate that in such a case the legacy MD5 image '
+                  'checksum should be used to validate the downloaded data. '
+                  'You can also set the enviroment variable '
+                  'OS_IMAGE_ALLOW_MD5_FALLBACK to any value to activate this '
+                  'option.'))
 @utils.arg('--file', metavar='<FILE>',
            help=_('Local file to save downloaded image data to. '
                   'If this is not specified and there is no redirection '
@@ -407,7 +530,8 @@ def do_image_download(gc, args):
         utils.exit(msg)
 
     try:
-        body = gc.images.data(args.id)
+        body = gc.images.data(args.id,
+                              allow_md5_fallback=args.allow_md5_fallback)
     except (exc.HTTPForbidden, exc.HTTPException) as e:
         msg = "Unable to download image '%s'. (%s)" % (args.id, e)
         utils.exit(msg)
@@ -435,8 +559,17 @@ def do_image_download(gc, args):
            help=_('Show upload progress bar.'))
 @utils.arg('id', metavar='<IMAGE_ID>',
            help=_('ID of image to upload data to.'))
+@utils.arg('--backend', metavar='<STORE>',
+           default=utils.env('OS_IMAGE_BACKEND', default=None),
+           help='Backend store to upload image to.')
 def do_image_upload(gc, args):
     """Upload data for a specific image."""
+    backend = None
+    if args.backend:
+        backend = args.backend
+        # determine if backend is valid
+        _validate_backend(backend, gc)
+
     image_data = utils.get_data_file(args)
     if args.progress:
         filesize = utils.get_file_size(image_data)
@@ -444,7 +577,7 @@ def do_image_upload(gc, args):
             # NOTE(kragniz): do not show a progress bar if the size of the
             # input is unknown (most likely a piped input)
             image_data = progressbar.VerboseFileWrapper(image_data, filesize)
-    gc.images.upload(args.id, image_data, args.size)
+    gc.images.upload(args.id, image_data, args.size, backend=backend)
 
 
 @utils.arg('--file', metavar='<FILE>',
@@ -481,13 +614,22 @@ def do_image_stage(gc, args):
            help=_('URI to download the external image.'))
 @utils.arg('id', metavar='<IMAGE_ID>',
            help=_('ID of image to import.'))
+@utils.arg('--backend', metavar='<STORE>',
+           default=utils.env('OS_IMAGE_BACKEND', default=None),
+           help='Backend store to upload image to.')
 def do_image_import(gc, args):
     """Initiate the image import taskflow."""
+    backend = None
+    if args.backend:
+        backend = args.backend
+        # determine if backend is valid
+        _validate_backend(backend, gc)
 
     if getattr(args, 'from_create', False):
         # this command is being called "internally" so we can skip
         # validation -- just do the import and get out of here
-        gc.images.image_import(args.id, args.import_method, args.uri)
+        gc.images.image_import(args.id, args.import_method, args.uri,
+                               backend=backend)
         return
 
     # do input validation
@@ -526,7 +668,8 @@ def do_image_import(gc, args):
                        "to an image in status 'queued'")
 
     # finally, do the import
-    gc.images.image_import(args.id, args.import_method, args.uri)
+    gc.images.image_import(args.id, args.import_method, args.uri,
+                           backend=backend)
 
     image = gc.images.get(args.id)
     utils.print_image(image)
@@ -607,16 +750,30 @@ def do_image_tag_delete(gc, args):
 @utils.arg('--metadata', metavar='<STRING>', default='{}',
            help=_('Metadata associated with the location. '
                   'Must be a valid JSON object (default: %(default)s)'))
+@utils.arg('--checksum', metavar='<STRING>',
+           help=_('md5 checksum of image contents'))
+@utils.arg('--hash-algo', metavar='<STRING>',
+           help=_('Multihash algorithm'))
+@utils.arg('--hash-value', metavar='<STRING>',
+           help=_('Multihash value'))
 @utils.arg('id', metavar='<IMAGE_ID>',
            help=_('ID of image to which the location is to be added.'))
 def do_location_add(gc, args):
     """Add a location (and related metadata) to an image."""
+    validation_data = {}
+    if args.checksum:
+        validation_data['checksum'] = args.checksum
+    if args.hash_algo:
+        validation_data['os_hash_algo'] = args.hash_algo
+    if args.hash_value:
+        validation_data['os_hash_value'] = args.hash_value
     try:
         metadata = json.loads(args.metadata)
     except ValueError:
         utils.exit('Metadata is not a valid JSON object.')
     else:
-        image = gc.images.add_location(args.id, args.url, metadata)
+        image = gc.images.add_location(args.id, args.url, metadata,
+                                       validation_data=validation_data)
         utils.print_dict(image)
 
 
